@@ -6,7 +6,7 @@ import click
 import logging
 import getpass
 
-from certvalidator import ValidationContext
+from pyhanko_certvalidator import ValidationContext
 from pyhanko.config import (
     init_validation_context_kwargs, parse_cli_config,
     CLIConfig, LogConfig, StdLogOutput, parse_logging_config
@@ -14,13 +14,12 @@ from pyhanko.config import (
 from pyhanko.pdf_utils import misc
 from pyhanko.pdf_utils.config_utils import ConfigurationError
 
-from pyhanko.sign import signers, pkcs11
+from pyhanko.sign import signers, validation, fields
 from pyhanko.sign.general import (
     SigningError, SignatureValidationError, load_certs_from_pemder
 )
 from pyhanko.sign.signers import DEFAULT_SIGNER_KEY_USAGE
 from pyhanko.sign.timestamps import HTTPTimeStamper
-from pyhanko.sign import validation, beid, fields
 from pyhanko.pdf_utils.reader import PdfFileReader
 from pyhanko.pdf_utils.writer import copy_into_new_writer
 from pyhanko.pdf_utils.incremental_writer import IncrementalPdfFileWriter
@@ -37,6 +36,13 @@ __all__ = ['cli']
 logger = logging.getLogger(__name__)
 
 
+try:
+    import pkcs11  # lgtm [py/unused-import]
+    pkcs11_available = True
+except ImportError:
+    pkcs11_available = False
+
+
 def logging_setup(log_configs):
     log_config: LogConfig
     for module, log_config in log_configs.items():
@@ -303,20 +309,20 @@ def _select_style(ctx, style_name, url):
 def _signature_status(ltv_profile, force_revinfo, soft_revocation_check,
                       pretty_print, vc_kwargs, key_usage_settings,
                       executive_summary, embedded_sig):
+    if soft_revocation_check and force_revinfo:
+        raise click.ClickException(
+            "--soft-revocation-check is incompatible with "
+            "--force-revinfo"
+        )
+    if force_revinfo:
+        rev_mode = 'require'
+    elif soft_revocation_check:
+        rev_mode = 'soft-fail'
+    else:
+        rev_mode = 'hard-fail'
+    vc_kwargs['revocation_mode'] = rev_mode
     try:
         if ltv_profile is None:
-            if soft_revocation_check and force_revinfo:
-                raise click.ClickException(
-                    "--soft-revocation-check is incompatible with "
-                    "--force-revinfo"
-                )
-            if force_revinfo:
-                rev_mode = 'require'
-            elif soft_revocation_check:
-                rev_mode = 'soft-fail'
-            else:
-                rev_mode = 'hard-fail'
-            vc_kwargs['revocation_mode'] = rev_mode
             vc = ValidationContext(**vc_kwargs)
             status = validation.validate_pdf_signature(
                 embedded_sig, key_usage_settings=key_usage_settings,
@@ -374,7 +380,7 @@ def _signature_status(ltv_profile, force_revinfo, soft_revocation_check,
               type=bool, is_flag=True, default=False, show_default=True)
 @click.option('--soft-revocation-check',
               help='Do not fail validation on revocation checking failures '
-                   '(meaningless for LTV validation)',
+                   '(only applied to on-line revocation checks)',
               type=bool, is_flag=True, default=False, show_default=True)
 @click.option('--no-revocation-check',
               help='Do not attempt to check revocation status '
@@ -426,15 +432,7 @@ def validate_signatures(ctx, infile, executive_summary,
                 "validating (for now)"
             )
 
-        # function to filter out timestamps
-        def is_sig(sig):
-            try:
-                return sig.sig_object['/Type'] == '/Sig'
-            except KeyError:
-                return True
-
-        sigs = filter(is_sig, r.embedded_signatures)
-        for ix, embedded_sig in enumerate(sigs):
+        for ix, embedded_sig in enumerate(r.embedded_regular_signatures):
             fingerprint: str = embedded_sig.signer_cert.sha256.hex()
             status_str = _signature_status(
                 ltv_profile, force_revinfo, soft_revocation_check,
@@ -526,9 +524,7 @@ def ltv_fix(ctx, infile, field, timestamp_url, apply_lta_timestamp,
 
     try:
         emb_sig = next(
-            s for s in r.embedded_signatures
-            if s.sig_object.get('/Type', None) == '/Sig'
-            and s.field_name == field
+            s for s in r.embedded_regular_signatures if s.field_name == field
         )
     except StopIteration:
         raise click.ClickException(
@@ -786,7 +782,6 @@ def addsig_pkcs12(ctx, infile, outfile, pfx, chain, passfile):
 
 # TODO add options to specify extra certs to include
 
-@addsig.command(name='pkcs11', help='use generic PKCS#11 device to sign')
 @click.argument('infile', type=click.File('rb'))
 @click.argument('outfile', type=click.File('wb'))
 @click.option('--lib', help='path to PKCS#11 module',
@@ -803,6 +798,7 @@ def addsig_pkcs12(ctx, infile, outfile, pfx, chain, passfile):
 @click.pass_context
 def addsig_pkcs11(ctx, infile, outfile, lib, token_label,
                   cert_label, key_label, slot_no, skip_user_pin):
+    from pyhanko.sign import pkcs11
     signature_meta = ctx.obj[Ctx.SIG_META]
     existing_fields_only = ctx.obj[Ctx.EXISTING_ONLY]
     timestamp_url = ctx.obj[Ctx.TIMESTAMP_URL]
@@ -836,7 +832,6 @@ def addsig_pkcs11(ctx, infile, outfile, lib, token_label,
         )
 
 
-@addsig.command(name='beid', help='use Belgian eID to sign')
 @click.argument('infile', type=click.File('rb'))
 @click.argument('outfile', type=click.File('wb'))
 @click.option('--lib', help='path to libbeidpkcs11 library file',
@@ -848,6 +843,7 @@ def addsig_pkcs11(ctx, infile, outfile, lib, token_label,
               required=False, type=int, default=None)
 @click.pass_context
 def addsig_beid(ctx, infile, outfile, lib, use_auth_cert, slot_no):
+    from pyhanko.sign import beid
     signature_meta = ctx.obj[Ctx.SIG_META]
     existing_fields_only = ctx.obj[Ctx.EXISTING_ONLY]
     timestamp_url = ctx.obj[Ctx.TIMESTAMP_URL]
@@ -870,6 +866,27 @@ def addsig_beid(ctx, infile, outfile, lib, use_auth_cert, slot_no):
         )
 
 
+def _pkcs11_cmd(name, hlp, fun):
+    addsig.command(name=name, help=hlp)(fun)
+
+
+PKCS11_COMMANDS = [
+    ('pkcs11', 'use generic PKCS#11 device to sign', addsig_pkcs11),
+    ('beid', 'use Belgian eID to sign', addsig_beid)
+]
+
+if pkcs11_available:
+    for args in PKCS11_COMMANDS:
+        _pkcs11_cmd(*args)
+else:
+    def _unavailable():
+        raise click.ClickException(
+            "This subcommand requires python-pkcs11 to be installed."
+        )
+    for name, hlp, fun in PKCS11_COMMANDS:
+        _pkcs11_cmd(name, hlp + ' [dependencies missing]', _unavailable)
+
+
 def _index_page(page):
     try:
         page_ix = int(page)
